import React from "react";
import PropTypes from "prop-types";
import styling from "@devcake/styling";
import "../init";
import { FormElement } from "../formElement";
import { Label } from "../Label";

styling.link("components.input.disabled", "components.form.disabled");
styling.setDefault(
  "components.input",
  "shadow-sm block w-full sm:text-sm rounded-md border-0"
);
styling.setDefault("components.input.container", "rounded-md border border-gray-200 focus-within:border-indigo-500 ring-0 focus-within:ring-1 ring-indigo-500");
styling.setDefault("components.input.with-inset", "rounded-r-none focus:ring-transparent");
styling.setDefault(
  "components.input.inset",
  "flex-grow-0 flex items-center pl-2 pr-3 shadow-sm rounded-r-md border-l pointer-events-none"
);

export type InputProps = FormElement & {
  inputType?: "email" | "text" | "password" | "tel";
  inset?: React.ReactNode;
};

export const Input = (props: InputProps) => {
  const { className, disabled, label, nullable, placeholder, inputType, inset } = props;
  const value = "value" in props ? props.value : "";
  const onChange = "onChange" in props ? props.onChange : () => null;

  const [innerValue, setInnerValue] = React.useState(value);

  // Update inner value when outer value changes
  React.useEffect(() => setInnerValue(value), [value]);

  let normalizedValue = innerValue;
  if (nullable && normalizedValue === null) normalizedValue = "";

  return (
    <>
      {label && <Label className="mb-1">{label}</Label>}
      <div className={["flex", styling.get("components.input.container")].join(" ")}>
        <input
          type={inputType}
          disabled={disabled}
          className={[
            className,
            disabled ? styling.get("components.input.disabled") : "",
            styling.get("components.input"),
            inset ? styling.get("components.input.with-inset") : ""
          ].join(" ")}
          placeholder={placeholder}
          value={normalizedValue}
          onChange={(evt) => {
            let { value: newValue } = evt.target;
            if (nullable && newValue == "") newValue = null;

            setInnerValue(newValue);
            if (onChange) onChange(newValue);
          }}
        />
        {inset && (
          <div className={styling.get("components.input.inset")}>
            {inset}
          </div>
        )}
      </div>
    </>
  );
};

Input.propTypes = {
  label: PropTypes.node,
  nullable: PropTypes.bool,
  value: PropTypes.string,
  disabled: PropTypes.bool,
  onChange: PropTypes.func,
  className: PropTypes.string,
  placeholder: PropTypes.string,
  inputType: PropTypes.oneOf(["text","password", "email", "tel"]),
  inset: PropTypes.node
};

Input.defaultProps = {
  disabled: false,
  nullable: true,
  inputType: "text"
};
